<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/woocommerce-ultimate-pricing/
 * @since      1.0.0
 *
 * @package    WooCommerce_delivery
 * @subpackage WooCommerce_delivery/public
 */
class WooCommerce_Ultimate_Pricing_Product_Discounts extends WooCommerce_Ultimate_Pricing {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	protected $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of this plugin.
	 */
	protected $version;

	/**
	 * options of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      array    $options
	 */
	protected $options;


	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) 
	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->data = array();
	}
	
	public function init()
	{
		global $woocommerce_ultimate_pricing_options;
		$this->options = $woocommerce_ultimate_pricing_options;

		if (!$this->get_option('enable')) {
			return false;
		}

		if (!$this->get_option('productDiscountsEnabled')) {
			return false;
		}

		if(is_admin() && !wp_doing_ajax()) { 
			return false;
		}

		$this->user_id = get_current_user_id();
		$this->discountsApplied = array();

		// Discount Filters
		add_filter( 'woocommerce_product_get_price', array($this, 'apply_discounts'), 30, 2);
		add_filter( 'woocommerce_product_get_sale_price', array($this, 'apply_discounts'), 30, 2);
		add_filter( 'woocommerce_product_variation_get_sale_price', array($this, 'apply_discounts'), 30, 2);
		add_filter( 'woocommerce_product_variation_get_price', array($this, 'apply_discounts'), 30, 2);

		add_filter( 'woocommerce_get_price_html', array($this, 'show_discount_badge_tooltip_in_shop'), 20, 2);
		add_filter( 'woocommerce_cart_item_price', array($this, 'show_discount_badge_tooltip_in_checkout'), 20, 3);

		// Variation price gets saved in transients ???
		add_filter( 'woocommerce_variation_prices_price', array($this, 'apply_discounts'), 30, 2);
		add_filter( 'woocommerce_variation_prices_sale_price', array($this, 'apply_discounts'), 30, 2);
	}

	public function apply_discounts($price, $product)
	{
		global $tieredPriceApplied, $productDiscountsApplied;

		if(!$product) {
			return $price;
		}

	    if(empty($price)) {
	    	return $price;
	    }

	    $productId = $product->get_id();

	    // ensures discounts do not override already discounted tiered prices
	    if($tieredPriceApplied && isset($tieredPriceApplied[$productId])){
	    	return $tieredPriceApplied[$productId]['price'];
	    }

		if(class_exists('WooCommerce_Reward_Points')) {
			$userCurrentLevel = 1;
			if($this->user_id > 0) { 
				$currentLevelMeta = get_user_meta($this->user_id, 'woocommerce_reward_points_current_level', true);
				if($currentLevelMeta) {
					$userCurrentLevel = (int) $currentLevelMeta;
				}
			} else {
				$currentLevelMeta = isset($_SESSION['woocommerce_reward_points_current_level']) ? $_SESSION['woocommerce_reward_points_current_level'] : false;
				if($currentLevelMeta) {
					$userCurrentLevel = (int) $currentLevelMeta;
				}
			}
		}

		if(!isset($this->discountsApplied[$productId])) {
			$this->discountsApplied[$productId] = array(
				'applied' => false,
				'rules' => array(),
				'fixedDiscountTotal' => 0,
				'percentageDiscountTotal' => 0,
				'discountRateTotal' => 0,
			);	
		}

		// Already applied rules - do not apply again performance
		if(isset($this->discountsApplied[$productId]['applied']) && $this->discountsApplied[$productId]['applied']) {
			return $this->discountsApplied[$productId]['newPrice'];
		}

		$percentageDiscountTotal = 0;
		$fixedDiscountTotal = 0;
		
		$userRoles = $this->get_user_roles($this->user_id);

		// Custom User Role Price
		if(!empty($userRoles) && $this->get_option('customUserRolePrices')) {
			$customUserRolePrices = $product->get_meta('woocommerce_ultimate_pricing_prices');

			if(!empty($customUserRolePrices)) {

				foreach($userRoles as $userRole) {
				 	if(isset($customUserRolePrices[$userRole]) && !empty($customUserRolePrices[$userRole])) {

						$originalPrice = $price;
						$newPrice = floatval( str_replace(',', '.', $customUserRolePrices[$userRole]) );
						$discountRateTotal = $originalPrice - $newPrice;

						$this->discountsApplied[$productId]['rules'][] = 'userRole';
						$this->discountsApplied[$productId]['applied'] = true;
						$this->discountsApplied[$productId]['newPrice'] = $newPrice;
						$this->discountsApplied[$productId]['oldPrice'] = $originalPrice;
						$this->discountsApplied[$productId]['fixedDiscountTotal'] = $discountRateTotal;
						$this->discountsApplied[$productId]['percentageDiscountTotal'] = $percentageDiscountTotal;

						// Will make sure old sale prices will be used for regular prices
						$product->set_regular_price($originalPrice);
						$this->discountsApplied[$productId]['discountRateTotal'] = $discountRateTotal;
						$this->discountsApplied[$productId]['discountPercentageTotal'] = round( ($originalPrice - $newPrice) / $originalPrice * 100, 2);

						return $newPrice;
					}
				}
			}
		}

		$B2BKingCustomergroup = get_user_meta($this->user_id, 'b2bking_customergroup', true);

		$productIdToCheckTaxonomies = $productId;
		if($product->is_type('variation')) {
			$productIdToCheckTaxonomies = $product->get_parent_id();
		}

		$productDiscountsCount = $this->get_option('productDiscountsCount');

		$productDiscountRules = array();
		for ($i=1; $i <= $productDiscountsCount; $i++) {

			if(!$this->get_option('productDiscountsRule' . $i . 'Enabled')) {
				continue;
			}

			$productDiscountRules[$i] = array(
				'id' => $i,
				'order' => $this->get_option('productDiscountsRule' . $i . 'Order'),
			);

		}

		$order = array_column($productDiscountRules, 'order');
		array_multisort($order, SORT_DESC, $productDiscountRules);

		foreach($productDiscountRules as $productDiscountRule) {

			$productDiscountRuleId = $productDiscountRule['id'];

		    $roles = $this->get_option('productDiscountsRule' . $productDiscountRuleId . 'Roles');
		    if(!empty($roles)) {

			   	// No Role or Guest
			   	if(!$userRoles) {
			   		continue;
			   	}

			   	$isIn = false;
			   	foreach($userRoles as $userRole) {
					if(in_array($userRole, $roles)) {
						$isIn = true;
						break;
					}
				}

				if(!$isIn) {
					continue;
				}
		    }

		    $B2BKingCustomerGroups = $this->get_option('productDiscountsRule' . $productDiscountRuleId . 'B2BKingCustomerGroups');
		    if(!empty($B2BKingCustomerGroups)) {

			   	if(!in_array($B2BKingCustomergroup, $B2BKingCustomerGroups)) {
			   		continue;
		   		}
		    }

		    $dateRange = $this->get_option('productDiscountsRule' . $productDiscountRuleId . 'DateRange');
		    if(!empty($dateRange)) {
		    	$dateRange = explode(' to ', $dateRange);
		    	$startDate = $dateRange[0];
		    	$endDate = $dateRange[1];
				if(!empty($startDate) && !empty($endDate)) {
					$currentDateObject = new DateTime();
					$startDateObject = new DateTime($startDate);
					$endDateObject = new DateTime($endDate . ' 23:59');
				    if($currentDateObject < $startDateObject || $currentDateObject > $endDateObject) {
				        continue;
				    }
				}
		    }
		    
		    $level = (int) $this->get_option('productDiscountsRule' . $productDiscountRuleId . 'Level');
			if($level > 0 && class_exists('WooCommerce_Reward_Points')) {
				if($userCurrentLevel < $level) {
					continue;
				}
			}

		    $categories = $this->get_option('productDiscountsRule' . $productDiscountRuleId . 'Categories');
		    if(!empty($categories)) {
			    if( !has_term($categories, 'product_cat', $productIdToCheckTaxonomies) ) {
			        continue;
			    }
		    }

		    $products = $this->get_option('productDiscountsRule' . $productDiscountRuleId . 'Products');
		    if(!empty($products)) {
			    if( !in_array($productIdToCheckTaxonomies, $products)) {
			        continue;
			    }
		    }

		    $tags = $this->get_option('productDiscountsRule' . $productDiscountRuleId . 'Tags');
		    if(!empty($tags)) {
			    if( !has_term($tags, 'product_tag', $productIdToCheckTaxonomies) ) {
			        continue;
			    }
		    }

		    $reviews = $this->get_option('productDiscountsRule' . $productDiscountRuleId . 'Reviews');
		    if(!empty($reviews)) {
			    $userReviews = $this->get_user_review_count($this->user_id);
			    if($userReviews < $reviews) {
			    	continue;
			    }
		    }

		    $orders = $this->get_option('productDiscountsRule' . $productDiscountRuleId . 'Orders');
		    if(!empty($orders)) {
			    $userOrders = $this->get_user_order_count($this->user_id);
			    if($userOrders < $orders) {
			    	continue;
			    }
		    }

		    $this->discountsApplied[$productId]['applied'] = true;

			if(!in_array($productDiscountRuleId, $this->discountsApplied[$productId]['rules'])) {
				$this->discountsApplied[$productId]['rules'][] = $productDiscountRuleId;
			}

		    if($this->get_option('productDiscountsRule' . $productDiscountRuleId . 'Addable')) {

				if($this->get_option('productDiscountsRule' . $productDiscountRuleId . 'Type') == "percentage") {

					$percentageDiscount = $this->get_option('productDiscountsRule' . $productDiscountRuleId . 'PercentageDiscount');
					if($this->get_option('productDiscountsRule' . $productDiscountRuleId . 'Multipliable')) {
						$percentageDiscountTotal = 100 - ($percentageDiscountTotal * $percentageDiscount / 100);
					} else {
						$percentageDiscountTotal = $percentageDiscountTotal + $percentageDiscount;
					}

				} else {
					$fixedDiscount = $this->get_option('productDiscountsRule' . $productDiscountRuleId . 'FixedDiscount');
					$fixedDiscountTotal = $fixedDiscountTotal + $fixedDiscount;
				}

	    	} else {

		    	$fixedDiscount = 0;
				$percentageDiscountTotal = 0;

				if($this->get_option('productDiscountsRule' . $productDiscountRuleId . 'Type') == "percentage") {
					$percentageDiscount = $this->get_option('productDiscountsRule' . $productDiscountRuleId . 'PercentageDiscount');
					$percentageDiscountTotal = $percentageDiscount;
				} else {
					$fixedDiscount = $this->get_option('productDiscountsRule' . $productDiscountRuleId . 'FixedDiscount');
					$fixedDiscountTotal = $fixedDiscount;
				}

				break;
	    	}
		}

		if($fixedDiscountTotal <= 0 && $percentageDiscountTotal <= 0) {
			return $price;
		}

		$this->discountsApplied[$productId]['fixedDiscountTotal'] = $fixedDiscountTotal;
		$this->discountsApplied[$productId]['percentageDiscountTotal'] = $percentageDiscountTotal;

		$originalPrice = $price;

		// Will make sure old sale prices will be used for regular prices
		$product->set_regular_price($originalPrice);

		if($percentageDiscountTotal > 0) {
			$newPrice = $price * ( 1 - intval($percentageDiscountTotal) / 100);
			if($newPrice <= 0) {
				$price = 0;
			} else {
				$price = $newPrice;
			}
		}

		if($fixedDiscountTotal > 0) {
			$newPrice = $price - floatval($fixedDiscountTotal);
			if($newPrice <= 0) {
				$price = 0;
			} else {
				$price = $newPrice;
			}
		}

		$this->discountsApplied[$productId]['discountRateTotal'] = $originalPrice - $newPrice;
		$this->discountsApplied[$productId]['discountPercentageTotal'] = round( ($originalPrice - $newPrice) / $originalPrice * 100, 2);
		$this->discountsApplied[$productId]['newPrice'] = $price;
		$this->discountsApplied[$productId]['oldPrice'] = $originalPrice;

		$productDiscountsApplied = $this->discountsApplied;

	   	return $price;
	}

	public function show_discount_badge_tooltip_in_shop($price_html, $product)
	{
		if(!$product) {
			return $price_html;	
		}

		$productId = $product->get_id();

		if(!isset($this->discountsApplied[$productId]) || empty($this->discountsApplied[$productId])) {
			return $price_html;
		}

		if(!isset($this->discountsApplied[$productId]['rules']) || empty($this->discountsApplied[$productId]['rules'])) {

			return $price_html;
		}

	    // Woo saves variation prices in a transient
	    // have to modify and rebuild original price_html code
	    // see: class-wc-product-variable.php > get_price_html
	    // if($product->is_type('variable')) {

		// 	$prices = $product->get_variation_prices( true );

		// 	if ( !empty( $prices['price'] ) ) {

		// 		if($this->discountsApplied[$productId]['percentageDiscountTotal'] > 0) {
		// 			$min_price     = current( $prices['price'] ) * ( 1 - intval($this->discountsApplied[$productId]['percentageDiscountTotal']) / 100);
		// 			$max_price     = end( $prices['price'] ) * (1 - intval($this->discountsApplied[$productId]['percentageDiscountTotal']) / 100);
		// 		}

		// 		if($this->discountsApplied[$productId]['fixedDiscountTotal'] > 0) {
		// 			$min_price     = current( $prices['price'] ) - floatval($this->discountsApplied[$productId]['fixedDiscountTotal']);
		// 			$max_price     = end( $prices['price'] ) - floatval($this->discountsApplied[$productId]['fixedDiscountTotal']);
		// 		}
				
		// 		$min_reg_price = current( $prices['regular_price'] );
		// 		$max_reg_price = end( $prices['regular_price'] );

		// 		if ( $min_price !== $max_price ) {
		// 			$price = wc_format_price_range( $min_price, $max_price );
		// 		} elseif ( $product->is_on_sale() && $min_reg_price === $max_reg_price ) {
		// 			$price = wc_format_sale_price( wc_price( $max_reg_price ), wc_price( $min_price ) );
		// 		} else {
		// 			$price = wc_price( $min_price );
		// 		}

		// 		$price_html = apply_filters( 'woocommerce_variable_price_html', $price . $product->get_price_suffix(), $product );

		// 		// On single products not show the discount applied stuff
		// 		// if(is_product()) {
		// 		// 	return $price_html;
		// 		// }
		// 	}
	    // } 

		$discountRulesTexts = array();
		$discountRuleTextTemplate = $this->get_option('productDiscountsTooltipSingleDiscountRuleText');

		foreach ($this->discountsApplied[$productId]['rules'] as $discountRuleId ) {

			$discountTooltipText = $this->get_option('productDiscountsRule' . $discountRuleId . 'TooltipText');
			if(!empty($discountTooltipText)) {
				$discountRulesTexts[] = $discountTooltipText;
			} else {
				$discountTitle = $this->get_option('productDiscountsRule' . $discountRuleId . 'Title');

				if(!empty($discountTitle)) {
					if( $this->get_option('productDiscountsRule' . $discountRuleId . 'Type') == "percentage") {
						$discountRate = intval( $this->get_option('productDiscountsRule' . $discountRuleId . 'PercentageDiscount') ) . '%';
					} else {
						$discountRate = wc_price( floatval( $this->get_option('productDiscountsRule' . $discountRuleId . 'FixedDiscount') ) );
					}

					$discountRulesTexts[] = str_replace(array('{{discountRate}}', '{{discountTitle}}'), array($discountRate, $discountTitle), $discountRuleTextTemplate);
				}
			}
		}

		$discountRulesText = "";
		if(count( $discountRulesTexts) > 1) {
			$discountRulesText = implode($this->get_option('productDiscountsTooltipDiscountRulesAppendix'), $discountRulesTexts);
		} elseif(!empty($discountRulesTexts)) {
			$discountRulesText = $discountRulesTexts[0];
		}
		
		// Show Badge
		if($this->get_option('productDiscountsBadgeEnabled')) {

			$text = $this->get_option('productDiscountsBadgeText');

			if(is_single()) {
				if(!in_array('singleProduct', $this->get_option('productDiscountsBadgeDisplay')) ) {
					$text = "";
				}
			} else {
				if(!in_array('productCategory', $this->get_option('productDiscountsBadgeDisplay')) ) {
					$text = "";
				}
			}	

			if(!empty($text)) {
		 		$text = str_replace(
			    	array(
						'{{fixedDiscountTotal}}',
						'{{percentageDiscountTotal}}',
						'{{discountRateTotal}}',
						'{{discountPercentageTotal}}',
						'{{discountRulesText}}',
						'{{oldPrice}}',
						'{{newPrice}}',
					),
			    	array(
						wc_price( $this->discountsApplied[$productId]['fixedDiscountTotal'] ),
						$this->discountsApplied[$productId]['percentageDiscountTotal'] . '%',
						wc_price( $this->discountsApplied[$productId]['discountRateTotal'] ),
						$this->discountsApplied[$productId]['discountPercentageTotal'] . '%',
						$discountRulesText,
						wc_price( $this->discountsApplied[$productId]['oldPrice'] ),
						wc_price( $this->discountsApplied[$productId]['newPrice'] ),
			    	),
			    	$text
			    );

		 		$tooltipText = $this->get_option('productDiscountsTooltipText');
			    if(!empty($tooltipText) && $this->get_option('productDiscountsBadgeTooltip') ) {

					$tooltipText = str_replace(
				    	array(
							'{{fixedDiscountTotal}}',
							'{{percentageDiscountTotal}}',
							'{{discountRateTotal}}',
							'{{discountPercentageTotal}}',
							'{{discountRulesText}}',
							'{{oldPrice}}',
							'{{newPrice}}',
						),
				    	array(
							wc_price( $this->discountsApplied[$productId]['fixedDiscountTotal'] ),
							$this->discountsApplied[$productId]['percentageDiscountTotal'] . '%',
							wc_price( $this->discountsApplied[$productId]['discountRateTotal'] ),
							$this->discountsApplied[$productId]['discountPercentageTotal'] . '%',
							$discountRulesText,
							wc_price( $this->discountsApplied[$productId]['oldPrice'] ),
							wc_price( $this->discountsApplied[$productId]['newPrice'] ),
				    	),
				    	$tooltipText
				    );

					$price_html .= '<span class="woocommerce-ultimate-pricing-tooltip" woocommerce-ultimate-pricing-tooltip="' . strip_tags( $tooltipText ) . '">' . '<span class="woocommerce-ultimate-pricing-badge">' . $text . '</span></span>';
			    } else {
			    	$price_html .= '<span class="woocommerce-ultimate-pricing-badge">' . $text . '</span>';
			    }

				
			}
		}

		// Show Tooltip
		if($this->get_option('productDiscountsTooltipEnabled') && $this->get_option('productDiscountsTooltipIconEnabled')) {

			$text = $this->get_option('productDiscountsTooltipText');

			if(is_single()) {
				if(!in_array('singleProduct', $this->get_option('productDiscountsTooltipDisplay')) ) {
					$text = "";
				}
			} else {
				if(!in_array('productCategory', $this->get_option('productDiscountsTooltipDisplay')) ) {
					$text = "";
				}
			}

			$tooltipIcon = $this->get_option('productDiscountsTooltipIcon');
			if(!empty($text) && isset($tooltipIcon['url']) && !empty($tooltipIcon['url'])) {

				$tooltipIconURL = $tooltipIcon['url'];
		 		$text = str_replace(
			    	array(
						'{{fixedDiscountTotal}}',
						'{{percentageDiscountTotal}}',
						'{{discountRateTotal}}',
						'{{discountPercentageTotal}}',
						'{{discountRulesText}}',
						'{{oldPrice}}',
						'{{newPrice}}',
					),
			    	array(
						wc_price( $this->discountsApplied[$productId]['fixedDiscountTotal'] ),
						$this->discountsApplied[$productId]['percentageDiscountTotal'] . '%',
						wc_price( $this->discountsApplied[$productId]['discountRateTotal'] ),
						$this->discountsApplied[$productId]['discountPercentageTotal'] . '%',
						$discountRulesText,
						wc_price( $this->discountsApplied[$productId]['oldPrice'] ),
						wc_price( $this->discountsApplied[$productId]['newPrice'] ),
			    	),
			    	$text
			    );

				$price_html .= '<span class="woocommerce-ultimate-pricing-tooltip" woocommerce-ultimate-pricing-tooltip="' . strip_tags( $text ) . '"><img width="' . $this->get_option('productDiscountsTooltipIconWidth') . 'px" class="woocommerce-ultimate-pricing-tooltip-icon" src="' . $tooltipIconURL . '"></span>';
			}
		}
	    
		$regularPrice = $product->get_regular_price();
	    if(!$product->is_on_sale() && !empty($regularPrice)) {

	    	if($product->is_type('variable')) {
	    		
	    		$minPrice = $product->get_variation_regular_price('min');
	    		$maxPrice = $product->get_variation_regular_price('max');

	    		if($minPrice != $maxPrice) {
	    			$regularPriceHTML = '<del>' . wc_price($minPrice) . ' – ' . wc_price($maxPrice) . '</del>';
	    		} else {
	    			$regularPriceHTML = '<del>' . wc_price($minPrice) . '</del>';
	    		}

	    	} else {
   				$regularPriceHTML = '<del>' . wc_price($regularPrice) . '</del>';
   			}

	    	$productDiscountsTextBeforeOriginalPrice = $this->get_option('productDiscountsTextBeforeOriginalPrice');
			$productDiscountsTextBeforeOriginalPriceText = $this->get_option('productDiscountsTextBeforeOriginalPriceText');
   			if($productDiscountsTextBeforeOriginalPrice && !empty($productDiscountsTextBeforeOriginalPriceText)) {
   				$regularPriceHTML = '<span class="woocommerce-ultimate-pricing-text-before-original-price">' . $productDiscountsTextBeforeOriginalPriceText . '</span>' . $regularPriceHTML;
   			}

			if($this->get_option('productDiscountsTextBeforeOriginalPriceLineBreak')) {
				$regularPriceHTML .= '<br>';
			}


	    	$productDiscountsTextBeforeDiscountedPrice = $this->get_option('productDiscountsTextBeforeDiscountedPrice');
			$productDiscountsTextBeforeDiscountedPriceText = $this->get_option('productDiscountsTextBeforeDiscountedPriceText');
   			if($productDiscountsTextBeforeDiscountedPrice && !empty($productDiscountsTextBeforeDiscountedPriceText)) {
   				$regularPriceHTML = $regularPriceHTML . ' <span class="woocommerce-ultimate-pricing-text-before-discounted-price">' . $productDiscountsTextBeforeDiscountedPriceText . '</span>';
   			}

   			$price_html =  $regularPriceHTML . $price_html;
   		}

	   	return $price_html;
	}

	public function show_discount_badge_tooltip_in_checkout($price_html, $cart_item, $cart_item_key ) 
	{
		$currentPage = 'cart';
		if(is_checkout()) {
			$currentPage = 'checkout';
		}

		if(!isset($cart_item['product_id']) || empty($cart_item['product_id'])) {
			return $price_html;
		}

		if(isset($cart_item['variation_id']) && !empty($cart_item['variation_id'])) {
			$product = wc_get_product($cart_item['variation_id']);	
		} else {
			$product = wc_get_product($cart_item['product_id']);	
		}
		
		if(!$product) {
			return $price_html;
		}

		$productId = $product->get_id();
		if(!isset($this->discountsApplied[$productId]) || empty($this->discountsApplied[$productId])) {
			return $price_html;
		}

		if(!isset($this->discountsApplied[$productId]['rules']) || empty($this->discountsApplied[$productId]['rules'])) {
			return $price_html;
		}

		$discountRulesTexts = array();
		$discountRuleTextTemplate = $this->get_option('productDiscountsTooltipSingleDiscountRuleText');

		foreach ($this->discountsApplied[$productId]['rules'] as $discountRuleId ) {

			$discountTooltipText = $this->get_option('productDiscountsRule' . $discountRuleId . 'TooltipText');
			if(!empty($discountTooltipText)) {
				$discountRulesTexts[] = $discountTooltipText;
			} else {
				$discountTitle = $this->get_option('productDiscountsRule' . $discountRuleId . 'Title');
				if( $this->get_option('productDiscountsRule' . $discountRuleId . 'Type') == "percentage") {
					$discountRate = intval( $this->get_option('productDiscountsRule' . $discountRuleId . 'PercentageDiscount') ) . '%';
				} else {
					$discountRate = wc_price( floatval( $this->get_option('productDiscountsRule' . $discountRuleId . 'FixedDiscount') ) );
				}

				$discountRulesTexts[] = str_replace(array('{{discountRate}}', '{{discountTitle}}'), array($discountRate, $discountTitle), $discountRuleTextTemplate);
			}
		}

		$discountRulesText = "";
		if(count( $discountRulesTexts) > 1) {
			$discountRulesText = implode($this->get_option('productDiscountsTooltipDiscountRulesAppendix'), $discountRulesTexts);
		} elseif(!empty($discountRulesTexts)) {
			$discountRulesText = $discountRulesTexts[0];
		}

		// Show Badge
		if($this->get_option('productDiscountsBadgeEnabled')) {

			$text = $this->get_option('productDiscountsBadgeText');

			if($currentPage == "cart" && !in_array('cart', $this->get_option('productDiscountsBadgeDisplay')) ) {
				$text = "";
			}

			if($currentPage == "checkout" && !in_array('checkout', $this->get_option('productDiscountsBadgeDisplay')) ) {
				$text = "";
			}

			if(!empty($text)) {
		 		$text = str_replace(
			    	array(
						'{{fixedDiscountTotal}}',
						'{{percentageDiscountTotal}}',
						'{{discountRateTotal}}',
						'{{discountPercentageTotal}}',
						'{{discountRulesText}}',
					),
			    	array(
						wc_price( $this->discountsApplied[$productId]['fixedDiscountTotal'] ),
						$this->discountsApplied[$productId]['percentageDiscountTotal'] . '%',
						wc_price( $this->discountsApplied[$productId]['discountRateTotal'] ),
						$this->discountsApplied[$productId]['discountPercentageTotal'] . '%',
						'',
			    	),
			    	$text
			    );

		 		$tooltipText = $this->get_option('productDiscountsTooltipText');
			    if(!empty($tooltipText) && $this->get_option('productDiscountsBadgeTooltip') ) {

					$tooltipText = str_replace(
				    	array(
							'{{fixedDiscountTotal}}',
							'{{percentageDiscountTotal}}',
							'{{discountRateTotal}}',
							'{{discountPercentageTotal}}',
							'{{discountRulesText}}',
						),
				    	array(
							wc_price( $this->discountsApplied[$productId]['fixedDiscountTotal'] ),
							$this->discountsApplied[$productId]['percentageDiscountTotal'] . '%',
							wc_price( $this->discountsApplied[$productId]['discountRateTotal'] ),
							$this->discountsApplied[$productId]['discountPercentageTotal'] . '%',
							$discountRulesText,
				    	),
				    	$tooltipText
				    );

				    $price_html .= '<span class="woocommerce-ultimate-pricing-tooltip" woocommerce-ultimate-pricing-tooltip="' . strip_tags( $tooltipText ) . '">' . '<span class="woocommerce-ultimate-pricing-badge">' . $text . '</span>' . '</span>';
			    } else {
			    	$price_html .= '<span class="woocommerce-ultimate-pricing-badge">' . $text . '</span>';
			    }
				
			}
		}

		// Show Tooltip
		// Woo strips svg etc.. not working
		if($this->get_option('productDiscountsTooltipEnabled') && $this->get_option('productDiscountsTooltipIconEnabled')) {

			$text = $this->get_option('productDiscountsTooltipText');

			if($currentPage == "cart" && !in_array('cart', $this->get_option('productDiscountsTooltipDisplay')) ) {
				$text = "";
			}

			if($currentPage == "checkout" && !in_array('checkout', $this->get_option('productDiscountsTooltipDisplay')) ) {
				$text = "";
			}

			$tooltipIcon = $this->get_option('productDiscountsTooltipIcon');
			if(!empty($text) && isset($tooltipIcon['url']) && !empty($tooltipIcon['url'])) {

				$tooltipIconURL = $tooltipIcon['url'];

		 		$text = str_replace(
			    	array(
						'{{fixedDiscountTotal}}',
						'{{percentageDiscountTotal}}',
						'{{discountRateTotal}}',
						'{{discountPercentageTotal}}',
						'{{discountRulesText}}',
					),
			    	array(
						wc_price( $this->discountsApplied[$productId]['fixedDiscountTotal'] ),
						$this->discountsApplied[$productId]['percentageDiscountTotal'] . '%',
						wc_price( $this->discountsApplied[$productId]['discountRateTotal'] ),
						$this->discountsApplied[$productId]['discountPercentageTotal'] . '%',
						$discountRulesText,
			    	),
			    	$text
			    );

				$price_html .= '<span class="woocommerce-ultimate-pricing-tooltip" woocommerce-ultimate-pricing-tooltip="' . strip_tags( $text ) . '"><img width="' . $this->get_option('productDiscountsTooltipIconWidth') . 'px" class="woocommerce-ultimate-pricing-tooltip-icon" src="' . $tooltipIconURL . '"></span>';
			}
		}

		return $price_html;
	}

}